package cn.com.qmth.stmms.biz.utils;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class TaskLockUtil {

    // 评卷任务并发处理互斥锁
    private static Map<String, TaskLock> formalTaskMap = new ConcurrentHashMap<>();

    // 试评任务并发处理互斥锁
    private static Map<String, TaskLock> trialTaskMap = new ConcurrentHashMap<>();

    public static void clearTimeoutTask(long timeoutMinute) {
        synchronized (TaskLockUtil.class) {
            if (formalTaskMap.size() > 0) {
                for (String key : formalTaskMap.keySet()) {
                    TaskLock taskLock = formalTaskMap.get(key);
                    if (taskLock != null) {
                        taskLock.expire(timeoutMinute);
                    }
                }
            }
            if (trialTaskMap.size() > 0) {
                for (String key : trialTaskMap.keySet()) {
                    TaskLock taskLock = trialTaskMap.get(key);
                    if (taskLock != null) {
                        taskLock.expire(timeoutMinute);
                    }
                }
            }
        }
    }

    public static TaskLock getTrialTask(String key) {
        synchronized (TaskLockUtil.class) {
            TaskLock taskLock = trialTaskMap.get(key);
            if (taskLock == null) {
                taskLock = new TaskLock();
                trialTaskMap.put(key, taskLock);
            }
            return taskLock;
        }
    }

    public static TaskLock getFormalTask(String key) {
        synchronized (TaskLockUtil.class) {
            TaskLock taskLock = formalTaskMap.get(key);
            if (taskLock == null) {
                taskLock = new TaskLock();
                formalTaskMap.put(key, taskLock);
            }
            return taskLock;
        }
    }

}
