package cn.com.qmth.stmms.admin.exam;

import cn.com.qmth.stmms.admin.dto.ExamQuestionDTO;
import cn.com.qmth.stmms.admin.thread.MarkGroupDeleteThread;
import cn.com.qmth.stmms.biz.exam.model.*;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.*;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.model.PictureConfigItem;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.*;
import cn.com.qmth.stmms.common.utils.RequestUtils;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JsonConfig;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;

import java.util.*;

@Controller("markGroupController")
@RequestMapping("/admin/exam/group")
public class MarkGroupController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(MarkGroupController.class);

    private static final String NULL_PAPER_TYPE_PLACEHOLDER = "#";

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkGroupService groupService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private MarkService markService;

    @Autowired
    private LockService lockService;

    @Autowired
    private ExamService examService;

    @Autowired
    private FileService fileService;

    @Qualifier("task-executor")
    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @Logging(menu = "大题管理查询", type = LogType.QUERY)
    @RequestMapping
    public String list(HttpServletRequest request, Model model, @RequestParam(required = false) String subjectCode) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        ExamSubject subject = null;
        if (StringUtils.isNotBlank(subjectCode)) {
            subject = subjectService.find(examId, subjectCode);
        } else if (!subjectList.isEmpty()) {
            subject = subjectList.get(0);
        }
        if (subject == null) {
            return "modules/exam/groupList";
        }
        List<MarkGroup> list = groupService.findByExamAndSubject(examId, subject.getCode());
        for (MarkGroup group : list) {
            group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                    subject.getCode(), false, group.getNumber()));
            group.setMarkerCount(markerService.countByExamAndSubjectAndGroup(examId, subject.getCode(),
                    group.getNumber()));
            group.setCurrentCount(markService.applyCount(group));
            int percent = group.getLibraryCount() > 0 ? (int) (group.getMarkedCount() * 100.00 / group
                    .getLibraryCount()) : 0;
            group.setPercent(percent);
            group.setDeleting(lockService.isLocked(LockType.GROUP_DELETE, group.getExamId(), group.getSubjectCode(),
                    group.getNumber()));
        }
        model.addAttribute("resultList", list);
        model.addAttribute("subject", subject);
        model.addAttribute("subjectList", subjectList);
        Exam exam = examService.findById(examId);
        model.addAttribute("examType", exam.getType());
        return "modules/exam/groupList";
    }

    @RequestMapping("/query")
    @ResponseBody
    public JSONArray query(HttpServletRequest request, @RequestParam String subjectCode,
            @RequestParam(required = false) Boolean withDouble, @RequestParam(required = false) MarkStatus status) {
        int examId = getSessionExamId(request);
        JSONArray array = new JSONArray();
        List<MarkGroup> list = withDouble != null && withDouble ? groupService.findByExamAndSubjectWithDouble(examId,
                subjectCode) : (status != null ? groupService
                .findByExamAndSubjectAndStatus(examId, subjectCode, status) : groupService.findByExamAndSubject(examId,
                subjectCode));
        for (MarkGroup group : list) {
            group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId, subjectCode,
                    false, group.getNumber()));
            JSONObject obj = new JSONObject();
            obj.accumulate("number", group.getNumber());
            obj.accumulate("title", group.getTitle());
            obj.accumulate("score", group.getTotalScore());
            array.add(obj);
        }
        return array;
    }

    @Logging(menu = "大题数量校对", type = LogType.UPDATE)
    @RequestMapping("/check-count")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String ckeckCount(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        ExamSubject subject = subjectService.find(examId, subjectCode);
        if (subject == null) {
            return "redirect:/admin/exam/mark";
        }
        markService.updateAllCount(examId, subjectCode);

        redirectAttributes.addAttribute("subjectCode", subjectCode);
        return "redirect:/admin/exam/group";
    }

    @Logging(menu = "大题任务回收", type = LogType.UPDATE)
    @RequestMapping("/release")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String release(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number) {
        int examId = getSessionExamId(request);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group == null) {
            return "redirect:/admin/exam/mark";
        }
        try {
            lockService.waitlock(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
            markService.releaseByGroup(group);
        } finally {
            lockService.unlock(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
        }
        redirectAttributes.addAttribute("subjectCode", subjectCode);
        return "redirect:/admin/exam/group";
    }

    @Logging(menu = "大题重置", type = LogType.UPDATE)
    @RequestMapping("/reset")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String reset(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number) {
        int examId = getSessionExamId(request);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group == null) {
            return "redirect:/admin/exam/mark";
        }
        try {
            lockService.waitlock(LockType.EXAM_SUBJECT, group.getExamId(), group.getSubjectCode());
            lockService.waitlock(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
            markService.resetByGroup(group);
        } catch (Exception e) {
            log.error("reset group error", e);
            throw new RuntimeException("重置大题失败", e);
        } finally {
            lockService.unlock(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
            lockService.unlock(LockType.EXAM_SUBJECT, group.getExamId(), group.getSubjectCode());
        }

        redirectAttributes.addAttribute("subjectCode", subjectCode);
        return "redirect:/admin/exam/group";
    }

    @Logging(menu = "大题状态修改", type = LogType.QUERY)
    @RequestMapping("/changeStatus")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String changeStatus(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number, @RequestParam MarkStatus status) {
        int examId = getSessionExamId(request);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group == null) {
            return "redirect:/admin/exam/mark";
        }
        try {
            lockService.waitlock(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
            boolean allow = false;
            if (group.getStatus() == MarkStatus.TRIAL && status == MarkStatus.FORMAL) {
                allow = true;
            } else if (group.getStatus() == MarkStatus.FORMAL && status == MarkStatus.FINISH
                    && group.getLeftCount() == 0) {
                allow = true;
            } else if (group.getStatus() == MarkStatus.FINISH && status == MarkStatus.FORMAL) {
                allow = true;
            }
            if (allow) {
                if (groupService.updateStatus(examId, subjectCode, number, status, group.getStatus())
                        && status == MarkStatus.FORMAL) {
                    // 切换到正评成功后刷新任务数量
                    group.setStatus(status);
                    markService.updateLibraryCount(group);
                    markService.updateMarkedCount(group);
                }
            } else {
                redirectAttributes.addAttribute("message", "不能切换到指定的评卷状态");
            }
        } finally {
            lockService.unlock(LockType.GROUP, group.getExamId(), group.getSubjectCode(), group.getNumber());
        }
        redirectAttributes.addAttribute("subjectCode", subjectCode);
        return "redirect:/admin/exam/group";
    }

    @RequestMapping("/add")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String add(HttpServletRequest request, Model model, @RequestParam String subjectCode) {
        int examId = getSessionExamId(request);
        ExamSubject subject = subjectService.find(examId, subjectCode);
        if (subject == null) {
            return "redirect:/admin/exam/mark";
        }
        MarkGroup group = new MarkGroup();
        group.setSubjectCode(subjectCode);
        model.addAttribute("group", group);
        model.addAttribute("markModeList", MarkMode.values());
        model.addAttribute("scorePolicyList", ScorePolicy.values());
        model.addAttribute("thirdPolicyList", ThirdPolicy.values());
        Exam exam = examService.findById(examId);
        model.addAttribute("examType", exam.getType());
        return "modules/exam/groupAdd";
    }

    @RequestMapping("/edit-simple")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String editSimple(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number) {
        int examId = getSessionExamId(request);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group != null) {
            String pictureConfig = buildPictureConfig(group);
            group.setPicList(pictureConfig);
            group.setQuestionList(questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId, subjectCode,
                    false, group.getNumber()));
            model.addAttribute("group", group);
            model.addAttribute(
                    "questions",
                    questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(group.getExamId(),
                            group.getSubjectCode(), false, group.getNumber()));
            model.addAttribute("pictureConfig", pictureConfig);
            model.addAttribute("markModeList", MarkMode.values());
            model.addAttribute("scorePolicyList", ScorePolicy.values());
            model.addAttribute("thirdPolicyList", ThirdPolicy.values());
            Exam exam = examService.findById(examId);
            model.addAttribute("examType", exam.getType());
            return "modules/exam/groupEditSimple";
        } else {
            redirectAttributes.addAttribute("subjectCode", subjectCode);
            return "redirect:/admin/exam/group";
        }
    }

    @RequestMapping("/edit-full")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String editFull(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number) {
        int examId = getSessionExamId(request);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group != null) {
            group.setPicList(buildPictureConfig(group));
            List<ExamQuestion> list = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(examId,
                    subjectCode, false, number);
            List<ExamQuestionDTO> questionList = new ArrayList<>();
            Map<Integer, ExamQuestionDTO> mainMap = new HashMap<>();
            for (ExamQuestion question : list) {
                ExamQuestionDTO dto = mainMap.get(question.getMainNumber());
                if (dto == null) {
                    dto = new ExamQuestionDTO();
                    dto.setMainNumber(question.getMainNumber());
                    dto.setMainTitle(question.getMainTitle());
                    dto.setSubNumber(question.getSubNumber());
                    dto.setScoreList(new ArrayList<>());
                    mainMap.put(question.getMainNumber(), dto);
                    questionList.add(dto);
                }
                dto.getScoreList().add(question.getTotalScore());
            }
            model.addAttribute("group", group);
            model.addAttribute("questionList", questionList);
            model.addAttribute("pictureConfig", group.getPicList());
            model.addAttribute("markModeList", MarkMode.values());
            model.addAttribute("scorePolicyList", ScorePolicy.values());
            model.addAttribute("thirdPolicyList", ThirdPolicy.values());
            Exam exam = examService.findById(examId);
            model.addAttribute("examType", exam.getType());
            return "modules/exam/groupEditFull";
        } else {
            redirectAttributes.addAttribute("subjectCode", subjectCode);
            return "redirect:/admin/exam/group";
        }
    }

    @Logging(menu = "删除大题", type = LogType.DELETE)
    @RequestMapping("/delete")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String delete(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number) {
        int examId = getSessionExamId(request);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group == null) {
            redirectAttributes.addAttribute("subjectCode", subjectCode);
            return "redirect:/admin/exam/group";
        }
        if (lockService.trylock(LockType.GROUP_DELETE, group.getExamId(), group.getSubjectCode(), group.getNumber())) {
            taskExecutor.submit(new MarkGroupDeleteThread(group, markService, lockService));
            RequestUtils.setLog(request, "开始删除分组，subjectCode：" + subjectCode + " number:" + number);
        }

        // try {
        // lockService.waitlock(LockType.EXAM_SUBJECT, group.getExamId(),
        // group.getSubjectCode());
        // lockService.waitlock(LockType.GROUP, group.getExamId(),
        // group.getSubjectCode(), group.getNumber());
        // markService.deleteGroup(group);
        // RequestUtils.setLog(request, "删除成功，subjectCode：" + subjectCode +
        // " number:" + number);
        // } catch (Exception e) {
        // log.error("delete group error", e);
        // throw new RuntimeException("删除大题失败", e);
        // } finally {
        // lockService.unlock(LockType.GROUP, group.getExamId(),
        // group.getSubjectCode(), group.getNumber());
        // lockService.unlock(LockType.EXAM_SUBJECT, group.getExamId(),
        // group.getSubjectCode());
        // }

        redirectAttributes.addAttribute("subjectCode", subjectCode);
        redirectAttributes.addAttribute("number", number);
        return "redirect:/admin/exam/group";
    }

    @Logging(menu = "修改大题", type = LogType.UPDATE)
    @SuppressWarnings("unchecked")
    @RequestMapping("/save")
    @RoleRequire(Role.SCHOOL_ADMIN)
    @Transactional
    public String save(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number, @RequestParam Boolean reset,
            @RequestParam(required = false) String picList, @RequestParam(required = false) Double doubleRate,
            @RequestParam(required = false) Double arbitrateThreshold,
            @RequestParam(required = false) Integer thirdPolicy, @RequestParam(required = false) Integer scorePolicy,
            @RequestParam(required = false) MarkMode markMode, @RequestParam(required = false) Integer trialCount,
            @RequestParam(required = false, defaultValue = "false") Boolean sheetView,
            @RequestParam(required = false, defaultValue = "false") Boolean enableAllZero,
            @RequestParam(required = false) String questionDetail,
            @RequestParam(required = false) String intervalScoreList) {
        int examId = getSessionExamId(request);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group != null) {
            try {
                if (questionDetail != null && reset.booleanValue()) {
                    // advance update
                    questionDetail = StringEscapeUtils.unescapeHtml(questionDetail);
                    JSONArray array = JSONArray.fromObject(questionDetail);
                    List<ExamQuestionDTO> detailList = JSONArray.toList(array, new ExamQuestionDTO(), new JsonConfig());
                    for (int i = 0; i < detailList.size(); i++) {
                        ExamQuestionDTO dto = detailList.get(i);
                        Object scoreListArray[] = array.getJSONObject(i).getJSONArray("scoreList").toArray();
                        List<Double> scoreList = new ArrayList<Double>();
                        for (int j = 0; j < scoreListArray.length; j++) {
                            scoreList.add(Double.parseDouble(scoreListArray[j].toString()));
                        }
                        dto.setScoreList(scoreList);
                    }
                    List<ExamQuestion> questionList = buildQuestionList(group, detailList);
                    List<ExamQuestion> others = questionService.findByExamAndSubjectAndObjectiveAndGroupNumberNotEqual(
                            examId, subjectCode, false, number);
                    Set<String> numbers = new HashSet<>();
                    Map<Integer, String> titles = new HashMap<>();
                    for (ExamQuestion question : others) {
                        numbers.add(question.getQuestionNumber());
                        titles.put(question.getMainNumber(), question.getMainTitle());
                    }
                    for (ExamQuestion question : questionList) {
                        if (numbers.contains(question.getQuestionNumber())) {
                            addMessage(redirectAttributes, "题号不能重复");
                            redirectAttributes.addAttribute("subjectCode", subjectCode);
                            redirectAttributes.addAttribute("number", number);
                            return "redirect:/admin/exam/group/edit-full";
                        }
                        if (titles.get(question.getMainNumber()) != null
                                && !titles.get(question.getMainNumber()).equals(question.getMainTitle())) {
                            addMessage(redirectAttributes, "大题名称不一致");
                            redirectAttributes.addAttribute("subjectCode", subjectCode);
                            redirectAttributes.addAttribute("number", number);
                            return "redirect:/admin/exam/group/edit-full";
                        }
                    }
                    if (!questionList.isEmpty()) {
                        ScorePolicy policy = scorePolicy != null ? ScorePolicy.findByValue(scorePolicy) : null;
                        ThirdPolicy third = thirdPolicy != null ? ThirdPolicy.findByValue(thirdPolicy)
                                : ThirdPolicy.DISABLE;
                        try {
                            lockService.waitlock(LockType.GROUP, true, group.getExamId(), group.getSubjectCode(),
                                    group.getNumber());
                            markService.updateGroup(group, questionList, policy, third);
                        } catch (Exception e) {
                            log.error("update group error", e);
                            throw new RuntimeException("重置更新大题失败", e);
                        } finally {
                            lockService.unlock(LockType.GROUP, true, group.getExamId(), group.getSubjectCode(),
                                    group.getNumber());
                        }
                    }
                } else {
                    // simple update
                    List<Double> intervalScores = buildDoubleList(intervalScoreList);
                    List<ExamQuestion> questionList = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(
                            examId, subjectCode, false, number);
                    if (intervalScores.size() == questionList.size()) {
                        for (int i = 0; i < questionList.size(); i++) {
                            ExamQuestion q = questionList.get(i);
                            q.setIntervalScore(intervalScores.get(i));
                            questionService.save(q);
                        }
                    }
                }
                // quick update
                picList = StringEscapeUtils.unescapeHtml(picList);
                JSONArray array = JSONArray.fromObject(picList);
                List<PictureConfigItem> list = JSONArray.toList(array, new PictureConfigItem(), new JsonConfig());
                if (list != null && !list.isEmpty()) {
                    groupService.updatePicList(examId, subjectCode, number, list);
                }
                if (doubleRate != null && doubleRate >= 0 && doubleRate <= 1) {
                    groupService.updateDoubleRate(examId, subjectCode, number, doubleRate);
                }
                if (arbitrateThreshold != null && arbitrateThreshold >= 0) {
                    groupService.updateArbitrateThreshold(examId, subjectCode, number, arbitrateThreshold);
                }
                groupService.updateMarkMode(examId, subjectCode, number, markMode);
                if (trialCount != null && trialCount > 0 && group.getStatus() == MarkStatus.TRIAL) {
                    groupService.updateTrialCount(examId, subjectCode, number, trialCount);
                }
                if (sheetView != null) {
                    groupService.updateSheetView(examId, subjectCode, number, sheetView);
                }
                if (enableAllZero != null) {
                    groupService.updateEnableAllZero(examId, subjectCode, number, enableAllZero);
                }
                redirectAttributes.addAttribute("subjectCode", subjectCode);
                redirectAttributes.addAttribute("number", number);
                return "redirect:/admin/exam/group";

            } catch (Exception e) {
                log.error("edit markgroup error", e);
                addMessage(redirectAttributes, "参数有误");
                redirectAttributes.addAttribute("subjectCode", subjectCode);
                redirectAttributes.addAttribute("number", number);
                return "redirect:/admin/exam/group";
            }
        } else {
            addMessage(redirectAttributes, "找不到对应大题");
            redirectAttributes.addAttribute("subjectCode", subjectCode);
            redirectAttributes.addAttribute("number", number);
            return "redirect:/admin/exam/group";
        }
    }

    @Logging(menu = "新增大题", type = LogType.ADD)
    @SuppressWarnings("unchecked")
    @RequestMapping("/insert")
    @RoleRequire(Role.SCHOOL_ADMIN)
    @Transactional
    public String insert(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number, @RequestParam String questionDetail,
            @RequestParam String picList, @RequestParam(required = false) Double doubleRate,
            @RequestParam(required = false) Double arbitrateThreshold,
            @RequestParam(required = false) Integer thirdPolicy, @RequestParam(required = false) Integer scorePolicy,
            @RequestParam(required = false) String markMode, @RequestParam(required = false) Integer trialCount,
            @RequestParam(required = false, defaultValue = "false") boolean sheetView,
            @RequestParam(defaultValue = "false") boolean enableAllZero) {
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        MarkGroup group = groupService.findOne(examId, subjectCode, number);
        if (group != null) {
            addMessage(redirectAttributes, "评卷分组序号不能重复");
            redirectAttributes.addAttribute("subjectCode", subjectCode);
            return "redirect:/admin/exam/group/add";
        } else if (StringUtils.isBlank(questionDetail)) {
            addMessage(redirectAttributes, "大题详情必须设置");
            redirectAttributes.addAttribute("subjectCode", subjectCode);
            return "redirect:/admin/exam/group/add";
        } else {
            try {
                // create group
                // build picList
                List<PictureConfigItem> picConfigList = null;
                if (!exam.getType().equals(ExamType.MULTI_MEDIA)) {
                    picList = StringEscapeUtils.unescapeHtml(StringUtils.trimToNull(picList));
                    JSONArray array = JSONArray.fromObject(picList);
                    picConfigList = JSONArray.toList(array, new PictureConfigItem(), new JsonConfig());
                }
                // build questionDetail
                questionDetail = StringEscapeUtils.unescapeHtml(questionDetail);
                JSONArray array = JSONArray.fromObject(questionDetail);
                List<ExamQuestionDTO> detailList = JSONArray.toList(array, new ExamQuestionDTO(), new JsonConfig());
                for (int i = 0; i < detailList.size(); i++) {
                    ExamQuestionDTO dto = detailList.get(i);
                    Object o[] = array.getJSONObject(i).getJSONArray("scoreList").toArray();
                    List<Double> scoreList = new ArrayList<Double>();
                    for (int j = 0; j < o.length; j++) {
                        scoreList.add(Double.parseDouble(o[j].toString()));
                    }
                    dto.setScoreList(scoreList);
                }
                List<ExamQuestion> current = questionService.findByExamAndSubjectAndObjectiveAndGroupNumberNotEqual(
                        examId, subjectCode, false, number);
                Set<String> numbers = new HashSet<>();
                Map<Integer, String> titles = new HashMap<>();
                for (ExamQuestion question : current) {
                    numbers.add(question.getQuestionNumber());
                    titles.put(question.getMainNumber(), question.getMainTitle());
                }
                if (detailList != null && detailList.size() > 0) {
                    group = new MarkGroup(examId, subjectCode, number, picConfigList, 0d, doubleRate,
                            arbitrateThreshold, scorePolicy, markMode, trialCount, sheetView, enableAllZero,
                            thirdPolicy);
                    List<ExamQuestion> list = buildQuestionList(group, detailList);
                    for (ExamQuestion question : list) {
                        if (numbers.contains(question.getQuestionNumber())) {
                            addMessage(redirectAttributes, "题号不能重复");
                            redirectAttributes.addAttribute("subjectCode", subjectCode);
                            return "redirect:/admin/exam/group/add";
                        }
                        if (titles.get(question.getMainNumber()) != null
                                && !titles.get(question.getMainNumber()).equals(question.getMainTitle())) {
                            addMessage(redirectAttributes, "大题名称不一致");
                            redirectAttributes.addAttribute("subjectCode", subjectCode);
                            return "redirect:/admin/exam/group/add";
                        }
                    }
                    // clear and replace exam_question
                    questionService
                            .deleteByExamAndSubjectAndObjectiveAndGroupNumber(examId, subjectCode, false, number);
                    questionService.save(list);
                    groupService.save(group);
                    subjectService.updateScore(examId, subjectCode, false,
                            groupService.sumTotalScore(examId, subjectCode));
                    studentService.updateSubjectiveStatusAndScoreAndInspectorId(examId, subjectCode,
                            SubjectiveStatus.UNMARK, 0, null, null, null);
                    redirectAttributes.addAttribute("subjectCode", subjectCode);
                    return "redirect:/admin/exam/group";
                } else {
                    addMessage(redirectAttributes, "参数有误");
                    redirectAttributes.addAttribute("subjectCode", subjectCode);
                    return "redirect:/admin/exam/group/add";
                }
            } catch (Exception e) {
                log.error("add markgroup error", e);
                addMessage(redirectAttributes, "参数有误");
                redirectAttributes.addAttribute("subjectCode", subjectCode);
                return "redirect:/admin/exam/group/add";
            }
        }
    }

    @RequestMapping("/getPictureConfig")
    @RoleRequire(Role.SCHOOL_ADMIN)
    public String get(HttpServletRequest request, Model model, RedirectAttributes redirectAttributes,
            @RequestParam String subjectCode, @RequestParam Integer number) {
        int examId = getSessionExamId(request);
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setUpload(true);
        List<ExamStudent> examStudents = studentService.findByQuery(query).getResult();
        MarkGroup group = null;
        if (number != null) {
            group = groupService.findOne(examId, subjectCode, number);
        }
        if (examStudents.size() > 0) {
            model.addAttribute("picUrls", buildPicUrl(examStudents.get(0)));
            model.addAttribute("pictureConfig", buildPictureConfig(group));
            model.addAttribute("fileServer", fileService.getFileServer());
            return "modules/mark/picConfig";
        } else {
            addMessage(redirectAttributes, "参数有误");
            redirectAttributes.addAttribute("subjectCode", subjectCode);
            return "redirect:/admin/exam/group/add";
        }
    }

    private String buildPictureConfig(MarkGroup group) {
        String json = "";
        if (group != null) {
            try {
                ObjectMapper mapper = new ObjectMapper();
                json = mapper.writeValueAsString(group.getPictureConfigList());
            } catch (JsonProcessingException e) {
                // e.printStackTrace();
            }
        }
        return json;
    }

    private List<String> buildPicUrl(ExamStudent student) {
        return fileService.getSliceUris(student.getExamId(), student.getSecretNumber(), 1, student.getSliceCount());
    }

    private List<ExamQuestion> buildQuestionList(MarkGroup group, List<ExamQuestionDTO> detailList) {
        List<ExamQuestion> list = new LinkedList<>();
        double totalScore = 0d;
        for (ExamQuestionDTO detail : detailList) {
            int i = 0;
            for (double score : detail.getScoreList()) {
                totalScore += score;
                ExamQuestion question = new ExamQuestion();
                question.setExamId(group.getExamId());
                question.setSubjectCode(group.getSubjectCode());
                question.setPaperType(NULL_PAPER_TYPE_PLACEHOLDER);
                question.setMainTitle(detail.getMainTitle());
                question.setMainNumber(detail.getMainNumber());
                question.setSubNumber(detail.getSubNumber() + i);
                question.setGroupNumber(group.getNumber());
                question.setObjective(false);
                question.setTotalScore(score);
                question.setIntervalScore(1d);
                list.add(question);
                i++;
            }
        }
        group.setTotalScore(totalScore);
        return list;
    }

    private List<Double> buildDoubleList(String content) {
        List<Double> list = new ArrayList<Double>();
        content = StringUtils.trimToNull(content);
        if (content != null) {
            String[] values = StringUtils.split(content, ",");
            if (values != null) {
                for (String value : values) {
                    try {
                        Double number = Double.valueOf(value);
                        if (number != null) {
                            list.add(number);
                        }
                    } catch (Exception e) {
                        continue;
                    }
                }
            }
        }
        return list;
    }
}
